from django.apps import AppConfig


class SimpleModerationConfig(AppConfig):
    name = "moderation"
    verbose_name = "Moderation"


class ModerationConfig(SimpleModerationConfig):
    def ready(self):
        # We have to import this here because it imports from models.py
        from .helpers import auto_discover
        auto_discover()



from __future__ import unicode_literals
from django_webtest import WebTest


class WebTestCase(WebTest):

    def setUp(self):
        self.user = None

    def get(self, url, **kwargs):
        kwargs.setdefault('user', self.user)
        return self.app.get(url, **kwargs)

    def post(self, url, **kwargs):
        kwargs.setdefault('user', self.user)
        return self.app.post(url, **kwargs)



from __future__ import unicode_literals

from django import VERSION
from django.contrib.auth.models import User, Group
from django.db import models
from django.test.testcases import TestCase
from django.test.utils import override_settings

from moderation.constants import (MODERATION_STATUS_REJECTED,
                                  MODERATION_STATUS_APPROVED,
                                  MODERATION_STATUS_PENDING,
                                  MODERATION_READY_STATE,
                                  MODERATION_DRAFT_STATE)
from moderation.fields import SerializedObjectField
from moderation.helpers import automoderate
from moderation.managers import ModerationObjectsManager
from moderation.models import ModeratedObject
from moderation.moderator import GenericModerator
from moderation.register import ModerationManager, RegistrationError
from tests.models import (UserProfile, SuperUserProfile, ModelWithSlugField2,
                          ProxyProfile)
from tests.utils import setup_moderation, teardown_moderation
from tests.utils import unittest


class SerializationTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        self.user = User.objects.get(username='moderator')
        self.profile = UserProfile.objects.get(user__username='moderator')

    def test_serialize_of_object(self):
        """Test if object is properly serialized to json"""

        json_field = SerializedObjectField()

        serialized_str = json_field._serialize(self.profile)

        self.assertIn('"pk": 1', serialized_str)
        self.assertIn('"model": "tests.userprofile"', serialized_str)
        self.assertIn('"fields": {', serialized_str)
        self.assertIn('"url": "http://www.google.com"', serialized_str)
        self.assertIn('"user": 1', serialized_str)
        self.assertIn('"description": "Old description"', serialized_str)

    def test_serialize_with_inheritance(self):
        """Test if object is properly serialized to json"""

        profile = SuperUserProfile(description='Profile for new super user',
                                   url='http://www.test.com',
                                   user=User.objects.get(username='user1'),
                                   super_power='invisibility')
        profile.save()
        json_field = SerializedObjectField()

        serialized_str = json_field._serialize(profile)

        self.assertIn('"pk": 2', serialized_str)
        self.assertIn('"model": "tests.superuserprofile"', serialized_str)
        self.assertIn('"fields": {"super_power": "invisibility"}',
                      serialized_str)
        self.assertIn('"pk": 2', serialized_str)
        self.assertIn('"model": "tests.userprofile"', serialized_str)
        self.assertIn('"url": "http://www.test.com"', serialized_str)
        self.assertIn('"user": 2', serialized_str)
        self.assertIn('"description": "Profile for new super user"',
                      serialized_str)
        self.assertIn('"fields": {', serialized_str)

    def test_deserialize(self):
        value = '[{"pk": 1, "model": "tests.userprofile", "fields": '\
                '{"url": "http://www.google.com", "user": 1, '\
                '"description": "Profile description"}}]'
        json_field = SerializedObjectField()
        object = json_field._deserialize(value)

        self.assertEqual(repr(object),
                         '<UserProfile: moderator - http://www.google.com>')
        self.assertTrue(isinstance(object, UserProfile))

    def test_deserialize_with_inheritance(self):
        value = '[{"pk": 2, "model": "tests.superuserprofile",'\
                ' "fields": {"super_power": "invisibility"}}, '\
                '{"pk": 2, "model": "tests.userprofile", "fields":'\
                ' {"url": "http://www.test.com", "user": 2,'\
                ' "description": "Profile for new super user"}}]'

        json_field = SerializedObjectField()
        object = json_field._deserialize(value)

        self.assertTrue(isinstance(object, SuperUserProfile))
        self.assertEqual(
            repr(object),
            '<SuperUserProfile: user1 - http://www.test.com - invisibility>')

    def test_deserialzed_object(self):
        moderated_object = ModeratedObject(content_object=self.profile)
        self.profile.description = 'New description'
        moderated_object.changed_object = self.profile
        moderated_object.save()
        pk = moderated_object.pk

        moderated_object = ModeratedObject.objects.get(pk=pk)

        self.assertEqual(moderated_object.changed_object.description,
                         'New description')

        self.assertEqual(moderated_object.content_object.description,
                         'Old description')

    def test_change_of_deserialzed_object(self):
        self.profile.description = 'New description'
        moderated_object = ModeratedObject(content_object=self.profile)
        moderated_object.save()
        pk = moderated_object.pk

        self.profile.description = 'New changed description'
        moderated_object.changed_object = self.profile.description
        moderated_object.save()

        moderated_object = ModeratedObject.objects.get(pk=pk)

        self.assertEqual(moderated_object.changed_object.description,
                         'New changed description')

    @unittest.skipIf(VERSION[:2] < (1, 4), "Proxy models require 1.4")
    def test_serialize_proxy_model(self):
        "Handle proxy models in the serialization."
        profile = ProxyProfile(description="I'm a proxy.",
                               url="http://example.com",
                               user=User.objects.get(username='user1'))
        profile.save()
        json_field = SerializedObjectField()

        serialized_str = json_field._serialize(profile)

        self.assertIn('"pk": 2', serialized_str)
        self.assertIn('"model": "tests.proxyprofile"', serialized_str)
        self.assertIn('"url": "http://example.com"', serialized_str)
        self.assertIn('"user": 2', serialized_str)
        self.assertIn('"description": "I\'m a proxy."', serialized_str)
        self.assertIn('"fields": {', serialized_str)

    @unittest.skipIf(VERSION[:2] < (1, 4), "Proxy models require 1.4")
    def test_deserialize_proxy_model(self):
        "Correctly restore a proxy model."
        value = '[{"pk": 2, "model": "tests.proxyprofile", "fields": '\
            '{"url": "http://example.com", "user": 2, '\
            '"description": "I\'m a proxy."}}]'

        json_field = SerializedObjectField()
        profile = json_field._deserialize(value)
        self.assertTrue(isinstance(profile, ProxyProfile))
        self.assertEqual(profile.url, "http://example.com")
        self.assertEqual(profile.description, "I\'m a proxy.")
        self.assertEqual(profile.user_id, 2)


class ModerateTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        self.user = User.objects.get(username='moderator')
        self.profile = UserProfile.objects.get(user__username='moderator')
        self.moderation = setup_moderation([UserProfile])

    def tearDown(self):
        teardown_moderation()

    def test_objects_with_no_moderated_object_are_visible(self):
        """
        Simulate conditions where moderation is added to a model which
        already has existing objects, which should remain visible.
        """

        ModeratedObject.objects.all().delete()

        self.assertEqual(
            [self.profile], list(self.profile.__class__.objects.all()),
            "Objects with no attached ModeratedObject should be visible "
            "by default.")

    def test_approval_status_pending(self):
        """test if before object approval status is pending"""

        self.profile.description = 'New description'
        self.profile.save()

        self.assertEqual(self.profile.moderated_object.status,
                         MODERATION_STATUS_PENDING)

    def test_moderate(self):
        self.profile.description = 'New description'
        self.profile.save()

        self.profile.moderated_object._moderate(MODERATION_STATUS_APPROVED,
                                                self.user, "Reason")

        self.assertEqual(self.profile.moderated_object.status,
                         MODERATION_STATUS_APPROVED)
        self.assertEqual(self.profile.moderated_object.by, self.user)
        self.assertEqual(self.profile.moderated_object.reason, "Reason")

    def test_multiple_moderations_throws_exception_by_default(self):
        self.profile.description = 'New description'
        self.profile.save()

        moderated_object = ModeratedObject.objects.create(
            content_object=self.profile)
        moderated_object.approve(by=self.user)

        with self.assertRaises(ModerationObjectsManager.MultipleModerations):
            self.profile.__class__.objects.get(id=self.profile.id)

    def test_approve_new_moderated_object(self):
        """
        When a newly created object is approved, it should become visible
        in standard querysets.
        """
        profile = self.profile.__class__(description='Profile for new user',
                                         url='http://www.test.com',
                                         user=self.user)

        profile.save()
        self.assertEqual(
            MODERATION_DRAFT_STATE, profile.moderated_object.state,
            "Before first approval, the profile should be in draft state, "
            "to hide it from querysets.")

        profile.moderated_object.approve(self.user)
        self.assertEqual(
            MODERATION_READY_STATE, profile.moderated_object.state,
            "After first approval, the profile should be in ready state, "
            "to show it in querysets.")

        user_profile = self.profile.__class__.objects.get(
            url='http://www.test.com')

        self.assertEqual(user_profile.description, 'Profile for new user')
        self.assertEqual(
            [profile],
            list(self.profile.__class__.objects.filter(pk=profile.pk)),
            "New profile objects should be visible after being accepted")

    def test_reject_new_moderated_object(self):
        """
        When a newly created object is rejected, it should remain invisible
        in standard querysets.
        """
        profile = self.profile.__class__(description='Profile for new user',
                                         url='http://www.test.com',
                                         user=self.user)

        profile.save()
        self.assertEqual(
            MODERATION_DRAFT_STATE, profile.moderated_object.state,
            "Before first approval, the profile should be in draft state, "
            "to hide it from querysets.")

        profile.moderated_object.reject(self.user)
        self.assertEqual(
            MODERATION_DRAFT_STATE, profile.moderated_object.state,
            "After rejection, the profile should still be in draft state, "
            "to hide it from querysets.")

        user_profile = self.profile.__class__.unmoderated_objects.get(
            url='http://www.test.com')
        self.assertEqual(user_profile.description, 'Profile for new user')
        self.assertEqual(
            [], list(self.profile.__class__.objects.filter(pk=profile.pk)),
            "New profile objects should be hidden after being rejected")

    def test_approve_modified_moderated_object(self):
        """
        When a previously approved object is updated and approved, it should
        remain visible in standard querysets, with the new data saved in the
        object.
        """
        self.profile.description = 'New description'
        self.profile.save()
        self.profile.moderated_object.approve(self.user)

        self.assertEqual(
            MODERATION_READY_STATE,
            self.profile.moderated_object.state,
            "After first approval, the profile should be in ready state, "
            "to show it in querysets.")

        user_profile = self.profile.__class__.objects.get(
            id=self.profile.id)
        self.assertEqual(user_profile.description, 'New description')
        self.assertEqual(
            [self.profile],
            list(self.profile.__class__.objects.filter(pk=self.profile.pk)),
            "Modified profile objects should be visible after being accepted")

    def test_reject_modified_moderated_object(self):
        """
        When a previously approved object is updated and rejected, it should
        remain visible in standard querysets, but with the old (previously
        approved) data saved in the object.
        """
        ModeratedObject.objects.create(content_object=self.profile)
        self.profile.moderated_object.approve(self.user)

        self.profile.description = 'New description'
        self.profile.save()

        self.profile.moderated_object.reject(self.user)
        self.assertEqual(
            MODERATION_READY_STATE,
            self.profile.moderated_object.state,
            "After rejection, the profile should still be in ready state, "
            "to show it in querysets, but with the old data.")

        user_profile = self.profile.__class__.objects.get(id=self.profile.id)

        self.assertEqual(user_profile.description, "Old description")
        self.assertEqual(self.profile.moderated_object.status,
                         MODERATION_STATUS_REJECTED)
        self.assertEqual(
            [self.profile],
            list(self.profile.__class__.objects.filter(pk=self.profile.pk)),
            "Modified profile objects should still be visible after being "
            "rejected, but with the old data")

        # Test making another change that's rejected, and that the data
        # saved in the object is still the previously approved data.
        self.profile.description = 'Another bad description'
        self.profile.save()

        self.profile.moderated_object.reject(self.user)

        user_profile = self.profile.__class__.objects.get(id=self.profile.id)

        self.assertEqual(user_profile.description, "Old description")
        self.assertEqual(self.profile.moderated_object.status,
                         MODERATION_STATUS_REJECTED)
        self.assertEqual(
            [self.profile],
            list(self.profile.__class__.objects.filter(pk=self.profile.pk)),
            "Modified profile objects should still be visible after being "
            "rejected, but with the old data")

    def test_has_object_been_changed_should_be_true(self):
        self.profile.description = 'Old description'
        moderated_object = ModeratedObject(content_object=self.profile)
        moderated_object.save()
        moderated_object.approve(by=self.user)

        user_profile = self.profile.__class__.objects.get(id=self.profile.id)

        self.profile.description = 'New description'
        moderated_object = ModeratedObject(content_object=self.profile)
        moderated_object.save()

        value = moderated_object.has_object_been_changed(user_profile)

        self.assertEqual(value, True)

    def test_has_object_been_changed_should_be_false(self):
        moderated_object = ModeratedObject(content_object=self.profile)
        moderated_object.save()

        value = moderated_object.has_object_been_changed(self.profile)

        self.assertEqual(value, False)


class AutoModerateTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        self.moderation = ModerationManager()

        class UserProfileModerator(GenericModerator):
            auto_approve_for_superusers = True
            auto_approve_for_staff = True
            auto_reject_for_groups = ['banned']

        self.moderation.register(UserProfile, UserProfileModerator)

        self.user = User.objects.get(username='moderator')
        self.profile = UserProfile.objects.get(user__username='moderator')

    def tearDown(self):
        teardown_moderation()

    def test_auto_approve_helper_status_approved(self):
        self.profile.description = 'New description'
        self.profile.save()

        status = automoderate(self.profile, self.user)
        self.assertEqual(status, MODERATION_STATUS_APPROVED)

        profile = UserProfile.objects.get(user__username='moderator')
        self.assertEqual(profile.description, 'New description')

    def test_auto_approve_helper_status_rejected(self):
        group = Group(name='banned')
        group.save()
        self.user.groups.add(group)
        self.user.save()

        self.profile.description = 'New description'
        self.profile.save()

        status = automoderate(self.profile, self.user)

        profile = UserProfile.unmoderated_objects.get(
            user__username='moderator')

        self.assertEqual(status, MODERATION_STATUS_REJECTED)
        self.assertEqual(profile.description, 'Old description')

    def test_model_not_registered_with_moderation(self):
        obj = ModelWithSlugField2(slug='test')
        obj.save()

        self.assertRaises(RegistrationError, automoderate, obj, self.user)


@unittest.skipIf(VERSION[:2] < (1, 5), "Custom auth users require 1.5")
# Using the decorator is causing problems with Django 1.3, so use
# the non-decorated version below.
# @override_settings(AUTH_USER_MODEL='tests.CustomUser')
class ModerateCustomUserTestCase(ModerateTestCase):

    def setUp(self):
        from tests.models import CustomUser,\
            UserProfileWithCustomUser
        from django.conf import settings
        self.user = CustomUser.objects.create(
            username='custom_user',
            password='aaaa')
        self.copy_m = ModeratedObject.by
        ModeratedObject.by = models.ForeignKey(
            getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
            blank=True, null=True, editable=False,
            related_name='moderated_objects')

        self.profile = UserProfileWithCustomUser.objects.create(
            user=self.user,
            description='Old description',
            url='http://google.com')
        self.moderation = setup_moderation([UserProfileWithCustomUser])

    def tearDown(self):
        teardown_moderation()
        ModeratedObject.by = self.copy_m

    # The actual tests are inherited from ModerateTestCase

if VERSION >= (1, 5):
    ModerateCustomUserTestCase = override_settings(
        AUTH_USER_MODEL='tests.CustomUser'
    )(ModerateCustomUserTestCase)


@unittest.skipIf(
    VERSION[:2] < (1, 5),
    "django.utils.six.with_metaclass does not work properly until 1.5"
)
class ModeratedModelTestCase(TestCase):

    def tearDown(self):
        teardown_moderation()

    def test_moderatedmodel_automatic_registration(self):
        from tests.more_models import MyTestModel
        from tests.more_models import MyTestModelWithoutModerator
        from moderation import moderation

        registered_models = moderation._registered_models
        # test registration with moderator
        moderator = registered_models.get(MyTestModel, None)
        is_registered = moderator is not None
        self.assertEqual(is_registered, True)
        # if Moderator extended then default notify_user should be overwritten
        notify_user = moderator.notify_user
        self.assertEqual(notify_user, False)
        # the value added to the Moderator should also show up
        made_up_value = moderator.made_up_value
        self.assertEqual(made_up_value, 'made_up')

        # test registration without custom moderator
        moderator = registered_models.get(MyTestModelWithoutModerator)
        self.assertEqual(
            moderator.__class__.__name__,
            'GenericModerator'
        )

    def test_django_14(self):
        # django_14 test
        from mock import patch, Mock
        from moderation.utils import django_14
        version = Mock()
        version.return_value = '1.4.8'
        with patch('django.get_version', version):
            self.assertTrue(django_14())



import django
from distutils.version import StrictVersion


def clear_builtins(attrs):
    """
    Clears the builtins from an ``attrs`` dict

    Returns a new dict without the builtins

    """
    new_attrs = {}

    for key in attrs.keys():
        if not(key.startswith('__') and key.endswith('__')):
            new_attrs[key] = attrs[key]

    return new_attrs


def django_19():
    if StrictVersion(django.get_version()) >= StrictVersion('1.9.0'):
        return True
    return False


def django_18():
    if StrictVersion(django.get_version()) >= StrictVersion('1.8.0'):
        return True
    return False


def django_17():
    if StrictVersion(django.get_version()) >= StrictVersion('1.7.0'):
        return True
    return False


def django_14():
    if StrictVersion(django.get_version()) < StrictVersion('1.5.0'):
        return True
    return False



from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.forms.models import ModelForm, model_to_dict

from .constants import (MODERATION_STATUS_PENDING, MODERATION_STATUS_REJECTED)
from .utils import django_17


class BaseModeratedObjectForm(ModelForm):
    class Meta:
        if django_17():
            exclude = '__all__'

    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)

        if instance:
            try:
                if instance.moderated_object.status in\
                   [MODERATION_STATUS_PENDING, MODERATION_STATUS_REJECTED] and\
                   not instance.moderated_object.moderator.\
                   visible_until_rejected:
                    initial = model_to_dict(
                        instance.moderated_object.changed_object)
                    kwargs.setdefault('initial', {})
                    kwargs['initial'].update(initial)
            except ObjectDoesNotExist:
                pass

        super(BaseModeratedObjectForm, self).__init__(*args, **kwargs)



from django import VERSION

from moderation import moderation
from example_project.example_app.models import ExampleUserProfile


from moderation.moderator import GenericModerator


class UserProfileModerator(GenericModerator):
    notify_user = False
    auto_approve_for_superusers = False
    auto_approve_for_staff = False


moderation.register(ExampleUserProfile)

if VERSION[:2] >= (1, 5):
    from example_project.example_app.models import UserProfileWithCustomUser
    moderation.register(UserProfileWithCustomUser, UserProfileModerator)



from __future__ import unicode_literals

try:
    from django.contrib.contenttypes.fields import GenericRelation
except ImportError:
    from django.contrib.contenttypes.generic import GenericRelation
from django.utils.six import with_metaclass

from .constants import (MODERATION_DRAFT_STATE,
                        MODERATION_STATUS_APPROVED,
                        MODERATION_STATUS_PENDING)
from .models import ModeratedObject, STATUS_CHOICES
from .moderator import GenericModerator


class RegistrationError(Exception):
    """Exception thrown when registration with Moderation goes wrong."""


class ModerationManagerSingleton(type):

    def __init__(cls, name, bases, dict):
        super(ModerationManagerSingleton, cls).__init__(name, bases, dict)
        cls.instance = None

    def __call__(cls, *args, **kw):
        if cls.instance is None:
            cls.instance = super(ModerationManagerSingleton, cls)\
                .__call__(*args, **kw)

        return cls.instance


class ModerationManager(with_metaclass(ModerationManagerSingleton, object)):
    def __init__(self, *args, **kwargs):
        """Initializes the moderation manager."""
        self._registered_models = {}

        super(ModerationManager, self).__init__(*args, **kwargs)

    def register(self, model_class, moderator_class=None):
        """Registers model class with moderation"""
        if model_class in self._registered_models:
            msg = "{} has been registered with Moderation.".format(model_class)
            raise RegistrationError(msg)
        if not moderator_class:
            moderator_class = GenericModerator

        if not issubclass(moderator_class, GenericModerator):
            msg = 'moderator_class must subclass '\
                  'GenericModerator class, found %s' % moderator_class
            raise AttributeError(msg)

        self._registered_models[model_class] = moderator_class(model_class)

        self._and_fields_to_model_class(self._registered_models[model_class])
        self._connect_signals(model_class)

    def _connect_signals(self, model_class):
        from django.db.models import signals

        signals.pre_save.connect(self.pre_save_handler,
                                 sender=model_class)
        signals.post_save.connect(self.post_save_handler,
                                  sender=model_class)

    def _add_moderated_object_to_class(self, model_class):
        if hasattr(model_class, '_relation_object'):
            relation_object = getattr(model_class, '_relation_object')
        else:
            relation_object = GenericRelation(
                ModeratedObject,
                object_id_field='object_pk')

        model_class.add_to_class('_relation_object', relation_object)

        def get_moderated_object(self):
            if not hasattr(self, '_moderated_object'):
                if self._relation_object.count() > 0:
                    self._moderated_object = getattr(self, '_relation_object')\
                        .filter().order_by('-updated')[0]
                else:
                    self._moderated_object = getattr(self, '_relation_object')\
                        .get()
            return self._moderated_object

        model_class.add_to_class('moderated_object',
                                 property(get_moderated_object))

    def _add_moderated_status_to_class(self, model_class):
        # Add the moderation_object to the class if it hasn't been yet
        if not hasattr(model_class, 'moderation_object'):
            self._add_moderated_object_to_class(model_class)

        def get_moderated_status(self):
            return STATUS_CHOICES[self.moderated_object.status]

        model_class.add_to_class('moderated_status',
                                 property(get_moderated_status))

    def _and_fields_to_model_class(self, moderator_class_instance):
        """Sets moderation manager on model class,
           adds generic relation to ModeratedObject,
           sets _default_manager on model class as instance of
           ModerationObjectsManager
        """
        model_class = moderator_class_instance.model_class
        base_managers = moderator_class_instance.base_managers
        moderation_manager_class = moderator_class_instance.\
            moderation_manager_class

        for manager_name, mgr_class in base_managers:
            ModerationObjectsManager = moderation_manager_class()(mgr_class)
            manager = ModerationObjectsManager()
            model_class.add_to_class('unmoderated_%s' % manager_name,
                                     mgr_class())
            model_class.add_to_class(manager_name, manager)

        self._add_moderated_object_to_class(model_class)
        self._add_moderated_status_to_class(model_class)

    def unregister(self, model_class):
        """Unregister model class from moderation"""
        try:
            moderator_instance = self._registered_models.pop(model_class)
        except KeyError:
            msg = "%r has not been registered with Moderation." % model_class
            raise RegistrationError(msg)

        self._remove_fields(moderator_instance)
        self._disconnect_signals(model_class)

    def _remove_fields(self, moderator_class_instance):
        """Removes fields from model class and disconnects signals"""

        model_class = moderator_class_instance.model_class
        base_managers = moderator_class_instance.base_managers

        for manager_name, manager_class in base_managers:
            manager = manager_class()
            delattr(model_class, 'unmoderated_%s' % manager_name)
            model_class.add_to_class(manager_name, manager)

        delattr(model_class, 'moderated_object')

    def _disconnect_signals(self, model_class):
        from django.db.models import signals

        signals.pre_save.disconnect(self.pre_save_handler, model_class)
        signals.post_save.disconnect(self.post_save_handler, model_class)

    def pre_save_handler(self, sender, instance, **kwargs):
        """
        Create a new moderation for the instance if it doesn't have any.
        If it does have a previous moderation, get that one
        """
        # check if object was loaded from fixture, bypass moderation if so
        if kwargs['raw']:
            return

        unchanged_obj = self._get_unchanged_object(instance)
        moderator = self.get_moderator(sender)
        if unchanged_obj:
            moderated_obj = self._get_or_create_moderated_object(instance,
                                                                 unchanged_obj,
                                                                 moderator)
            if not (moderated_obj.status ==
                    MODERATION_STATUS_APPROVED or
                    moderator.bypass_moderation_after_approval):
                moderated_obj.save()

    def _get_unchanged_object(self, instance):
        if instance.pk is None:
            return None
        pk = instance.pk
        try:
            unchanged_obj = instance.__class__._default_manager.get(pk=pk)
            return unchanged_obj
        except instance.__class__.DoesNotExist:
            return None

    def _get_updated_object(self, instance, unchanged_obj, moderator):
        """
        Returns the unchanged object with the excluded fields updated to
        those from the instance.
        """
        excludes = moderator.fields_exclude
        for field in instance._meta.fields:
            if field.name in excludes:
                value = getattr(instance, field.name)
                setattr(unchanged_obj, field.name, value)

        return unchanged_obj

    def _get_or_create_moderated_object(self, instance,
                                        unchanged_obj, moderator):
        """
        Get or create ModeratedObject instance.
        If moderated object is not equal instance then serialize unchanged
        in moderated object in order to use it later in post_save_handler
        """
        def get_new_instance(unchanged_obj):
            moderated_object = ModeratedObject(content_object=unchanged_obj)
            moderated_object.changed_object = unchanged_obj
            return moderated_object

        try:
            # if moderator.keep_history:
            #     moderated_object = get_new_instance(unchanged_obj)
            # else:
            #     moderated_object = ModeratedObject.objects.\
            #         get_for_instance(instance)

            moderated_object = ModeratedObject.objects.get_for_instance(
                instance)
            if moderated_object is None:
                moderated_object = get_new_instance(unchanged_obj)
            elif moderator.keep_history and \
                    moderated_object.has_object_been_changed(
                    instance):
                # We're keeping history and this isn't an update of an existing
                # moderation
                moderated_object = get_new_instance(unchanged_obj)

        except ModeratedObject.DoesNotExist:
            moderated_object = get_new_instance(unchanged_obj)

        else:
            if moderated_object.has_object_been_changed(instance):
                if moderator.visible_until_rejected:
                    moderated_object.changed_object = instance
                else:
                    moderated_object.changed_object = self._get_updated_object(
                        instance, unchanged_obj, moderator)
            elif moderated_object.has_object_been_changed(instance,
                                                          only_excluded=True):
                moderated_object.changed_object = self._get_updated_object(
                    instance, unchanged_obj, moderator)

        return moderated_object

    def get_moderator(self, model_class):
        try:
            moderator_instance = self._registered_models[model_class]
        except KeyError:
            msg = "%r has not been registered with Moderation." % model_class
            raise RegistrationError(msg)

        return moderator_instance

    def post_save_handler(self, sender, instance, **kwargs):
        """
        Creates new moderation object if instance is created,
        If instance exists and is only updated then save instance as
        content_object of moderated_object
        """
        # check if object was loaded from fixture, bypass moderation if so

        if kwargs['raw']:
            return

        pk = instance.pk
        moderator = self.get_moderator(sender)

        if kwargs['created']:
            old_object = sender._default_manager.get(pk=pk)
            moderated_obj = ModeratedObject(content_object=old_object)
            if not moderator.visible_until_rejected:
                # Hide it by placing in draft state
                moderated_obj.state = MODERATION_DRAFT_STATE
            moderated_obj.save()
            moderator.inform_moderator(instance)
            return

        moderated_obj = ModeratedObject.objects.get_for_instance(instance)

        if (moderated_obj.status == MODERATION_STATUS_APPROVED and
                moderator.bypass_moderation_after_approval):
            # save new data in moderated object
            moderated_obj.changed_object = instance
            moderated_obj.save()
            return

        if moderated_obj.has_object_been_changed(instance):
            copied_instance = self._copy_model_instance(instance)

            if not moderator.visible_until_rejected:
                # Save instance with old data from changed_object, undoing
                # the changes that save() just saved to the database.
                moderated_obj.changed_object.save_base(raw=True)

                # Save the new data in moderated_object, so it will be applied
                # to the real record when the moderator approves the change.
                moderated_obj.changed_object = copied_instance

            moderated_obj.status = MODERATION_STATUS_PENDING
            moderated_obj.save()
            moderator.inform_moderator(instance)
            instance._moderated_object = moderated_obj

    def _copy_model_instance(self, obj):
        initial = dict(
            [(f.name, getattr(obj, f.name)) for f in obj._meta.fields])
        return obj.__class__(**initial)



from datetime import datetime

from django.contrib.contenttypes.models import ContentType
from django.db.models.query import QuerySet

from . import moderation
from .constants import (MODERATION_READY_STATE,
                        MODERATION_STATUS_REJECTED,
                        MODERATION_STATUS_APPROVED)
from .signals import post_many_moderation, pre_many_moderation


class ModeratedObjectQuerySet(QuerySet):
    def approve(self, cls, by, reason=None):
        self._send_signals_and_moderate(cls, MODERATION_STATUS_APPROVED, by, reason)

    def reject(self, cls, by, reason=None):
        self._send_signals_and_moderate(cls, MODERATION_STATUS_REJECTED, by, reason)

    def moderator(self, cls):
        return moderation.get_moderator(cls)

    def _send_signals_and_moderate(self, cls, new_status, by, reason):
        pre_many_moderation.send(sender=cls,
                                 queryset=self,
                                 status=new_status,
                                 by=by,
                                 reason=reason)

        self._moderate(cls, new_status, by, reason)

        post_many_moderation.send(sender=cls,
                                  queryset=self,
                                  status=new_status,
                                  by=by,
                                  reason=reason)

    def _moderate(self, cls, new_status, by, reason):
        mod = self.moderator(cls)
        ct = ContentType.objects.get_for_model(cls)

        update_kwargs = {
            'status': new_status,
            'on': datetime.now(),
            'by': by,
            'reason': reason,
        }
        if new_status == MODERATION_STATUS_APPROVED:
            update_kwargs['state'] = MODERATION_READY_STATE

        self.update(update_kwargs)

        if mod.visibility_column:
            if new_status == MODERATION_STATUS_APPROVED:
                new_visible = True
            elif new_status == MODERATION_STATUS_REJECTED:
                new_visible = False
            else:  # MODERATION_STATUS_PENDING
                new_visible = mod.visibile_until_rejected

            cls.objects.filter(
                id__in=self.filter(content_type=ct)
                           .values_list('object_id', flat=True))\
               .update(**{mod.visibility_column: new_visible})

        mod.inform_users(self.exclude(changed_by=None).select_related('changed_by__email'))



from __future__ import unicode_literals

from django.db.models import Count, Q
from django.db.models.manager import Manager
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned

from . import moderation
from .constants import MODERATION_READY_STATE
from .queryset import ModeratedObjectQuerySet
from .utils import django_17, django_18


class MetaClass(type(Manager)):

    def __new__(cls, name, bases, attrs):
        return super(MetaClass, cls).__new__(cls, name, bases, attrs)


class ModerationObjectsManager(Manager):
    class MultipleModerations(Exception):
        def __init__(self, base_object):
            self.base_object = base_object
            super(ModerationObjectsManager.MultipleModerations,
                  self).__init__(
                "Multiple moderations found for object/s: %s" %
                base_object)

    def __call__(self, base_manager, *args, **kwargs):
        return MetaClass(
            self.__class__.__name__,
            (self.__class__, base_manager),
            {'use_for_related_fields': True})

    if django_18():
        def filter_moderated_objects(self, queryset):
            # Find any objects that have more than one related ModeratedObject
            annotated_queryset = queryset\
                .annotate(num_moderation_objects=Count('_relation_object'))\
                .filter(num_moderation_objects__gt=1)

            if annotated_queryset.exists():
                # No sensible default action here. You need to override
                # filter_moderated_objects() to handle this as you see fit.
                raise self.MultipleModerations(annotated_queryset)

            filter_kwargs = {
                '_relation_object__state': MODERATION_READY_STATE,
            }

            return queryset.filter(Q(**{'_relation_object': None}) | Q(**filter_kwargs))

    else:
        # Django < 1.7 doesn't properly annotate using GenericRelation fields,
        # so we keep a copy of the old code, even though it runs N+1 queries
        def filter_moderated_objects(self, query_set):
            # We have to import this here to avoid a circular import between
            # .models and .managers
            from .models import ModeratedObject

            exclude_pks = []

            mobjs_set = ModeratedObject.objects.filter(
                content_type=ContentType.objects.get_for_model(query_set.model),
                object_pk__in=query_set.values_list('pk', flat=True))

            # TODO: Load this query in chunks to avoid huge RAM usage spikes
            mobjects = {}
            for mobject in mobjs_set:
                if mobject.object_pk in mobjects:
                    # No sensible default action here. You need to override
                    # filter_moderated_objects() to handle this as you see fit.
                    raise self.MultipleModerations(mobject)
                else:
                    mobjects[mobject.object_pk] = mobject

            full_query_set = super(ModerationObjectsManager, self).get_queryset()\
                .filter(pk__in=query_set.values_list('pk', flat=True))

            for obj in full_query_set:
                try:
                    # We cannot use dict.get() here!
                    mobject = mobjects[obj.pk] if obj.pk in mobjects else \
                        obj.moderated_object

                    if mobject.state != MODERATION_READY_STATE:
                        exclude_pks.append(obj.pk)
                except ObjectDoesNotExist:
                    pass

            return query_set.exclude(pk__in=exclude_pks)

    def exclude_objs_by_visibility_col(self, query_set):
        return query_set.exclude(**{self.moderator.visibility_column: False})

    def get_queryset(self):
        query_set = None

        try:
            query_set = super(ModerationObjectsManager, self).get_queryset()
        except AttributeError:
            query_set = super(ModerationObjectsManager, self).get_query_set()

        if self.moderator.visibility_column:
            return self.exclude_objs_by_visibility_col(query_set)

        return self.filter_moderated_objects(query_set)

    if not django_17():
        get_query_set = get_queryset
        del get_queryset

    @property
    def moderator(self):
        return moderation.get_moderator(self.model)


class ModeratedObjectManager(Manager):
    def get_queryset(self):
        return ModeratedObjectQuerySet(self.model, using=self._db)

    if not django_17():
        get_query_set = get_queryset
        del get_queryset

    def get_for_instance(self, instance):
        '''Returns ModeratedObject for given model instance'''
        try:
            moderated_object = self.get(object_pk=instance.pk,
                                        content_type=ContentType.objects
                                        .get_for_model(instance.__class__))
        except self.model.MultipleObjectsReturned:
            # Get the most recent one
            moderated_object = self.filter(object_pk=instance.pk,
                                           content_type=ContentType.objects
                                           .get_for_model(instance.__class__))\
                .order_by('-updated')[0]
        return moderated_object



"""
    This module enables automatic Model registration with custom Moderators

    usage example:

        class MyModel(ModeratedModel):
            desc = models.TextField()

            class Moderator:
                notify_user = False

"""
import inspect

from django.db.models import base
from django.utils.six import with_metaclass

from .moderator import GenericModerator
from .register import ModerationManager
from .utils import clear_builtins, django_14

moderation = ModerationManager()


class ModeratedModelBase(type):
    """
    Metaclass for the ``ModeratedModel`` type

        -- automatically registers ``ModeratedModel's``
        -- resolves subclass ``Moderator`` into
           a instance of ``GenericModerator``

    """
    def _resolve_moderator(cls):
        """
        ``ModeratedModel`` that defines the class Moderator
        will have that class resolved into
        a class derived from ``GenericModerator``

        usage example:

        class MyModel(ModeratedModel):
            desc = models.TextField()

            # ``Moderator`` below will extend ``GenericModerator``
            # and will be used when the ``Model`` is registered
            class Moderator:
                notify_user = False

        """
        if hasattr(cls, 'Moderator') and inspect.isclass(cls.Moderator):
            Moderator = cls.Moderator
            # in python3 __dict__ is dictproxy
            attrs = dict(Moderator.__dict__)
            attrs = clear_builtins(attrs)

            return type(
                '%sModerator' % cls.__name__,
                (GenericModerator,),
                attrs,
            )
        else:
            return None

    def __init__(cls, name, bases, clsdict):
        """
        Registers ``ModeratedModel``

        """
        super(ModeratedModelBase, cls).__init__(name, bases, clsdict)

        if (any(x.__name__ == 'ModeratedModel' for x in cls.mro()[1:])):
            moderation.register(cls, cls._resolve_moderator())


class ModelBase(ModeratedModelBase, base.ModelBase):
    """
    Common metaclass for ``ModeratedModel`` enabling it to inherit
    the behavior of django ``Model`` objects

    """

if not django_14():
    # django.utils.six.with_metaclass is broken in django < 1.5
    class ModeratedModel(with_metaclass(ModelBase, base.Model)):
        class Meta:
            abstract = True



from django.contrib import admin
from django import VERSION

from example_project.example_app.models import ExampleUserProfile

from moderation.admin import ModerationAdmin


class ExampleUserProfileAdmin(ModerationAdmin):
    pass


class UserProfileWithCustomUserAdmin(ModerationAdmin):
    pass


admin.site.register(ExampleUserProfile, ExampleUserProfileAdmin)

if VERSION[:2] >= (1, 5):
    from example_project.example_app.models import UserProfileWithCustomUser, \
        CustomUser

    admin.site.register(UserProfileWithCustomUser,
                        UserProfileWithCustomUserAdmin)

    from django import forms
    from django.contrib import admin
    from django.contrib.auth.admin import UserAdmin
    from django.contrib.auth.forms import ReadOnlyPasswordHashField

    class CustomUserCreationForm(forms.ModelForm):
        """A form for creating new users. Includes all the required
        fields, plus a repeated password."""
        password1 = forms.CharField(label='Password',
                                    widget=forms.PasswordInput)
        password2 = forms.CharField(label='Password confirmation',
                                    widget=forms.PasswordInput)

        class Meta:
            model = CustomUser
            fields = ('username', 'email', 'date_of_birth', )

        def clean_password2(self):
            # Check that the two password entries match
            password1 = self.cleaned_data.get("password1")
            password2 = self.cleaned_data.get("password2")
            if password1 and password2 and password1 != password2:
                raise forms.ValidationError("Passwords don't match")
            return password2

        def save(self, commit=True):
            # Save the provided password in hashed format
            user = super(CustomUserCreationForm, self).save(commit=False)
            user.set_password(self.cleaned_data["password1"])
            if commit:
                user.save()
            return user

    class UserChangeForm(forms.ModelForm):
        """A form for updating users. Includes all the fields on
        the user, but replaces the password field with admin's
        password hash display field.
        """
        password = ReadOnlyPasswordHashField()

        class Meta:
            model = CustomUser

        def clean_password(self):
            # Regardless of what the user provides, return the initial value.
            # This is done here, rather than on the field, because the
            # field does not have access to the initial value
            return self.initial["password"]

    class MyUserAdmin(UserAdmin):
        # The forms to add and change user instances
        form = UserChangeForm
        add_form = CustomUserCreationForm

        # The fields to be used in displaying the User model.
        # These override the definitions on the base UserAdmin
        # that reference specific fields on auth.User.
        list_display = ('username', 'email', 'date_of_birth', 'is_staff')
        list_filter = ('is_staff',)
        fieldsets = (
            (None, {'fields': ('username', 'email', 'password')}),
            ('Personal info', {'fields': ('date_of_birth',)}),
            ('Permissions', {'fields': ('is_staff',)}),
            ('Important dates', {'fields': ('last_login',)}),
        )
        # add_fieldsets is not a standard ModelAdmin attribute. UserAdmin
        # overrides get_fieldsets to use this attribute when creating a user.
        add_fieldsets = (
            (None, {'classes': ('wide',), 'fields': (
                'username', 'email', 'date_of_birth', 'password1', 'password2'
            )}
            ),
        )
        search_fields = ('email',)
        ordering = ('email',)
        filter_horizontal = ()

    # Now register the new UserAdmin...
    admin.site.register(CustomUser, MyUserAdmin)



from django.db import models
from django.conf import settings
from django import VERSION


class ExampleUserProfile(models.Model):
    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'))
    description = models.TextField()
    url = models.URLField()

    def __unicode__(self):
        return "%s - %s" % (self.user, self.url)

    def get_absolute_url(self):
        return '/test/'


if VERSION[:2] >= (1, 5):
    from django.contrib.auth.models import AbstractUser

    class CustomUser(AbstractUser):
        date_of_birth = models.DateField(blank=True, null=True)
        height = models.FloatField(blank=True, null=True)

    class UserProfileWithCustomUser(models.Model):
        user = models.ForeignKey(
            getattr(settings, 'AUTH_USER_MODEL', 'auth.User'))
        description = models.TextField()
        url = models.URLField()

        def __unicode__(self):
            return "%s - %s" % (self.user, self.url)

        def get_absolute_url(self):
            return '/test/'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

from django.conf import settings
from django.db import models, migrations

from ..fields import SerializedObjectField


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModeratedObject',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_pk', models.PositiveIntegerField(null=True, editable=False, blank=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_updated', models.DateTimeField(default=datetime.datetime.now, auto_now=True)),
                ('moderation_state', models.SmallIntegerField(default=1, editable=False, choices=[(0, 'Ready for moderation'), (1, 'Draft')])),
                ('moderation_status', models.SmallIntegerField(default=2, editable=False, choices=[(1, 'Approved'), (2, 'Pending'), (0, 'Rejected')])),
                ('moderation_date', models.DateTimeField(null=True, editable=False, blank=True)),
                ('moderation_reason', models.TextField(null=True, blank=True)),
                ('changed_object', SerializedObjectField(editable=False)),
                ('changed_by', models.ForeignKey(related_name='changed_by_set', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('content_type', models.ForeignKey(blank=True, editable=False, to='contenttypes.ContentType', null=True)),
                ('moderated_by', models.ForeignKey(related_name='moderated_by_set', blank=True, editable=False, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'ordering': ['moderation_status', 'date_created'],
            },
            bases=(models.Model,),
        ),
    ]



default_app_config = "moderation.apps.ModerationConfig"


class _ModerationProxy(object):
    """Proxy the ModerationManager()

    It must not be created here for Django >= 1.8, because __init__ is invoked
    too early, but must be present once a module registers its models.
    """
    _moderation = None

    def _ensure_obj(self):
        if _ModerationProxy._moderation is None:
            from .register import ModerationManager
            _ModerationProxy._moderation = ModerationManager()

    def __getattr__(self, attribute):
        self._ensure_obj()
        return getattr(_ModerationProxy._moderation, attribute)

    def __setattr__(self, attribute, value):
        self._ensure_obj()
        return setattr(_ModerationProxy._moderation, attribute, value)

moderation = _ModerationProxy()



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
try:
    from django.contrib.auth import get_user_model
except ImportError: # django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()

USER_MODEL = "%s.%s" % (User._meta.app_label, User._meta.object_name)

class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'ModeratedObject.date_updated'
        db.add_column('moderation_moderatedobject', 'date_updated',
                      self.gf('django.db.models.fields.DateTimeField')(blank=True, auto_now=True, default=datetime.datetime.now),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'ModeratedObject.date_updated'
        db.delete_column('moderation_moderatedobject', 'date_updated')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '80', 'unique': 'True'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'symmetrical': 'False', 'to': "orm['auth.Permission']"})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission', 'ordering': "('content_type__app_label', 'content_type__model', 'codename')"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        # this should replace "auth.user"
        "%s.%s" % (User._meta.app_label, User._meta.model_name): {
        'Meta': {'object_name': User.__name__},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'", 'ordering': "('name',)"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'moderation.moderatedobject': {
            'Meta': {'object_name': 'ModeratedObject', 'ordering': "['moderation_status', 'date_created']"},
            'changed_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'changed_by_set'", 'blank': 'True', 'null': 'True', 'to': "orm['%s']" % USER_MODEL}),
            'changed_object': ('moderation.fields.SerializedObjectField', [], {}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'blank': 'True', 'auto_now': 'True', 'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'moderated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'moderated_by_set'", 'blank': 'True', 'null': 'True', 'to': "orm['%s']" % USER_MODEL}),
            'moderation_date': ('django.db.models.fields.DateTimeField', [], {'blank': 'True', 'null': 'True'}),
            'moderation_reason': ('django.db.models.fields.TextField', [], {'blank': 'True', 'null': 'True'}),
            'moderation_state': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'moderation_status': ('django.db.models.fields.SmallIntegerField', [], {'default': '2'}),
            'object_pk': ('django.db.models.fields.PositiveIntegerField', [], {'blank': 'True', 'null': 'True'})
        }
    }

    complete_apps = ['moderation']


from __future__ import unicode_literals

import warnings

from django.conf import settings


MODERATORS = getattr(settings, "DJANGO_MODERATION_MODERATORS", ())
if MODERATORS:
    warnings.warn(
        "`%s` is deprecated, use `%s` instead." %
        ("DJANGO_MODERATION_MODERATORS", "MODERATION_MODERATORS"))
MODERATORS = getattr(settings, "MODERATION_MODERATORS", ())

SOUTH_MIGRATION_MODULES = getattr(settings, "SOUTH_MIGRATION_MODULES", {})
SOUTH_MIGRATION_MODULES['moderation'] = 'moderation.migrations-pre17'



from __future__ import unicode_literals

import django
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.core import urlresolvers
from django.forms.models import ModelForm
from django.utils.translation import ugettext as _

from . import moderation
from .constants import (MODERATION_STATUS_REJECTED,
                        MODERATION_STATUS_APPROVED,
                        MODERATION_STATUS_PENDING)
from .diff import get_changes_between_models
try:
    from .filterspecs import RegisteredContentTypeListFilter
except ImportError:
    # Django < 1.4
    available_filters = ('content_type', 'status')
else:
    # Django >= 1.4
    available_filters = (
        ('content_type', RegisteredContentTypeListFilter), 'status')
from .forms import BaseModeratedObjectForm
from .helpers import automoderate
from .models import ModeratedObject
from .utils import django_17


def approve_objects(modeladmin, request, queryset):
    for obj in queryset:
        obj.approve(by=request.user)

approve_objects.short_description = _("Approve selected moderated objects")


def reject_objects(modeladmin, request, queryset):
    for obj in queryset:
        obj.reject(by=request.user)

reject_objects.short_description = _("Reject selected moderated objects")


def set_objects_as_pending(modeladmin, request, queryset):
    queryset.update(status=MODERATION_STATUS_PENDING)

set_objects_as_pending.short_description = _("Set selected moderated"
                                             " objects as Pending")


class ModerationAdmin(admin.ModelAdmin):
    admin_integration_enabled = True

    def get_form(self, request, obj=None, **kwargs):
        if obj and self.admin_integration_enabled:
            self.form = self.get_moderated_object_form(obj.__class__)

        return super(ModerationAdmin, self).get_form(request, obj, **kwargs)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        if self.admin_integration_enabled:
            self.send_message(request, object_id)

        try:
            return super(ModerationAdmin, self)\
                .change_view(request, object_id, form_url=form_url,
                             extra_context=extra_context)
        except TypeError:
            return super(ModerationAdmin, self)\
                .change_view(request, object_id, extra_context=extra_context)

    def send_message(self, request, object_id):
        try:
            obj = self.model.unmoderated_objects.get(pk=object_id)
            moderated_obj = ModeratedObject.objects.get_for_instance(obj)
            moderator = moderated_obj.moderator
            msg = self.get_moderation_message(moderated_obj.status,
                                              moderated_obj.reason,
                                              moderator.visible_until_rejected)
        except ModeratedObject.DoesNotExist:
            msg = self.get_moderation_message()

        self.message_user(request, msg)

    def save_model(self, request, obj, form, change):
        obj.save()
        automoderate(obj, request.user)

    def get_moderation_message(self, status=None, reason=None,
                               visible_until_rejected=False):
        if status == MODERATION_STATUS_PENDING:
            if visible_until_rejected:
                return _("Object is viewable on site, "
                         "it will be removed if moderator rejects it")
            else:
                return _("Object is not viewable on site, "
                         "it will be visible if moderator accepts it")
        elif status == MODERATION_STATUS_REJECTED:
            return _("Object has been rejected by moderator, "
                     "reason: %s" % reason)
        elif status == MODERATION_STATUS_APPROVED:
            return _("Object has been approved by moderator "
                     "and is visible on site")
        elif status is None:
            return _("This object is not registered with "
                     "the moderation system.")

    def get_moderated_object_form(self, model_class):

        class ModeratedObjectForm(BaseModeratedObjectForm):

            class Meta:
                model = model_class
                if django_17():
                    fields = '__all__'

        return ModeratedObjectForm


class ModeratedObjectAdmin(admin.ModelAdmin):
    date_hierarchy = 'created'
    list_display = ('content_object', 'content_type', 'created',
                    'status', 'by', 'on')
    list_filter = available_filters
    change_form_template = 'moderation/moderate_object.html'
    change_list_template = 'moderation/moderated_objects_list.html'
    actions = [reject_objects, approve_objects, set_objects_as_pending]
    fieldsets = (
        ('Object moderation', {'fields': ('reason',)}),
    )

    def get_actions(self, request):
        actions = super(ModeratedObjectAdmin, self).get_actions(request)
        # Remove the delete_selected action if it exists
        try:
            del actions['delete_selected']
        except KeyError:
            pass
        return actions

    def content_object(self, obj):
        return str(obj.changed_object)

    def get_moderated_object_form(self, model_class):

        class ModeratedObjectForm(ModelForm):

            class Meta:
                model = model_class
                if django_17():
                    fields = '__all__'

        return ModeratedObjectForm

    def change_view(self, request, object_id, extra_context=None):
        moderated_object = ModeratedObject.objects.get(pk=object_id)

        changed_obj = moderated_object.changed_object

        moderator = moderation.get_moderator(changed_obj.__class__)

        if moderator.visible_until_rejected:
            old_object = changed_obj
            new_object = moderated_object.get_object_for_this_type()
        else:
            old_object = moderated_object.get_object_for_this_type()
            new_object = changed_obj

        changes = list(get_changes_between_models(
            old_object,
            new_object,
            moderator.fields_exclude,
            resolve_foreignkeys=moderator.resolve_foreignkeys).values())

        if request.POST:
            admin_form = self.get_form(request, moderated_object)(request.POST)

            if admin_form.is_valid():
                reason = admin_form.cleaned_data['reason']
                if 'approve' in request.POST:
                    moderated_object.approve(request.user, reason)
                elif 'reject' in request.POST:
                    moderated_object.reject(request.user, reason)

        content_type = ContentType.objects.get_for_model(changed_obj.__class__)
        try:
            object_admin_url = urlresolvers.reverse("admin:%s_%s_change" %
                                                    (content_type.app_label,
                                                     content_type.model),
                                                    args=(changed_obj.pk,))
        except urlresolvers.NoReverseMatch:
            object_admin_url = None

        extra_context = {'changes': changes,
                         'django_version': django.get_version()[:3],
                         'object_admin_url': object_admin_url}
        return super(ModeratedObjectAdmin, self).change_view(
            request,
            object_id,
            extra_context=extra_context)


admin.site.register(ModeratedObject, ModeratedObjectAdmin)



from __future__ import unicode_literals

from django.conf import settings
try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models, transaction
from django.utils.translation import ugettext_lazy as _

from model_utils import Choices

from . import moderation
from .constants import (MODERATION_READY_STATE,
                        MODERATION_DRAFT_STATE,
                        MODERATION_STATUS_REJECTED,
                        MODERATION_STATUS_APPROVED,
                        MODERATION_STATUS_PENDING)
from .diff import get_changes_between_models
from .fields import SerializedObjectField
from .managers import ModeratedObjectManager
from .signals import post_moderation, pre_moderation
from .utils import django_19

import datetime


MODERATION_STATES = Choices(
    (MODERATION_READY_STATE, 'ready', _('Ready for moderation')),
    (MODERATION_DRAFT_STATE, 'draft', _('Draft')),
)

STATUS_CHOICES = Choices(
    (MODERATION_STATUS_REJECTED, 'rejected', _("Rejected")),
    (MODERATION_STATUS_APPROVED, 'approved', _("Approved")),
    (MODERATION_STATUS_PENDING, 'pending', _("Pending")),
)


class ModeratedObject(models.Model):
    content_type = models.ForeignKey(ContentType, null=True, blank=True,
                                     editable=False)
    object_pk = models.PositiveIntegerField(null=True, blank=True,
                                            editable=False)
    content_object = GenericForeignKey(ct_field="content_type",
                                       fk_field="object_pk")
    created = models.DateTimeField(auto_now_add=True, editable=False)
    updated = models.DateTimeField(auto_now=True)
    state = models.SmallIntegerField(choices=MODERATION_STATES,
                                     default=MODERATION_DRAFT_STATE,
                                     editable=False)
    status = models.SmallIntegerField(
        choices=STATUS_CHOICES,
        default=MODERATION_STATUS_PENDING,
        editable=False)
    by = models.ForeignKey(
        getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
        blank=True, null=True, editable=False,
        related_name='moderated_objects')
    on = models.DateTimeField(editable=False, blank=True, null=True)
    reason = models.TextField(blank=True, null=True)
    changed_object = SerializedObjectField(serialize_format='json',
                                           editable=False)
    changed_by = models.ForeignKey(
        getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
        blank=True, null=True, editable=True,
        related_name='changed_by_set')

    objects = ModeratedObjectManager()

    content_type.content_type_filter = True

    def __init__(self, *args, **kwargs):
        self.instance = kwargs.get('content_object')
        super(ModeratedObject, self).__init__(*args, **kwargs)

    def __unicode__(self):
        return "%s" % self.changed_object

    def __str__(self):
        return "%s" % self.changed_object

    def save(self, *args, **kwargs):
        if self.instance:
            self.changed_object = self.instance

        super(ModeratedObject, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _('Moderated Object')
        verbose_name_plural = _('Moderated Objects')
        ordering = ['status', 'created']

    def automoderate(self, user=None):
        '''Auto moderate object for given user.
          Returns status of moderation.
        '''
        if user is None:
            user = self.changed_by
        else:
            self.changed_by = user
            # No need to save here, both reject() and approve() will save us.
            # Just save below if the moderation result is PENDING.

        if self.moderator.visible_until_rejected:
            changed_object = self.get_object_for_this_type()
        else:
            changed_object = self.changed_object
        status, reason = self._get_moderation_status_and_reason(
            changed_object,
            user)

        if status == MODERATION_STATUS_REJECTED:
            self.reject(by=self.by, reason=reason)
        elif status == MODERATION_STATUS_APPROVED:
            self.approve(by=self.by, reason=reason)
        else:  # MODERATION_STATUS_PENDING
            self.save()

        return status

    def _get_moderation_status_and_reason(self, obj, user):
        '''
        Returns tuple of moderation status and reason for auto moderation
        '''
        reason = self.moderator.is_auto_reject(obj, user)
        if reason:
            return MODERATION_STATUS_REJECTED, reason
        else:
            reason = self.moderator.is_auto_approve(obj, user)
            if reason:
                return MODERATION_STATUS_APPROVED, reason

        return MODERATION_STATUS_PENDING, None

    def get_object_for_this_type(self):
        pk = self.object_pk
        obj = self.content_type.model_class()._default_manager.get(pk=pk)
        return obj

    def get_absolute_url(self):
        if hasattr(self.changed_object, 'get_absolute_url'):
            return self.changed_object.get_absolute_url()
        return None

    def get_admin_moderate_url(self):
        if django_19():
            return "/admin/moderation/moderatedobject/%s/change/" % self.pk
        else:
            return "/admin/moderation/moderatedobject/%s/" % self.pk

    @property
    def moderator(self):
        model_class = self.content_object.__class__

        return moderation.get_moderator(model_class)

    def _send_signals_and_moderate(self, new_status, by, reason):
        pre_moderation.send(sender=self.changed_object.__class__,
                            instance=self.changed_object,
                            status=new_status)

        self._moderate(new_status, by, reason)

        post_moderation.send(sender=self.content_object.__class__,
                             instance=self.content_object,
                             status=new_status)

    def _moderate(self, new_status, by, reason):
        # See register.py pre_save_handler() for the case where the model is
        # reset to its old values, and the new values are stored in the
        # ModeratedObject. In such cases, on approval, we should restore the
        # changes to the base object by saving the one attached to the
        # ModeratedObject.

        if (self.status == MODERATION_STATUS_PENDING and
                new_status == MODERATION_STATUS_APPROVED and
                not self.moderator.visible_until_rejected):
            base_object = self.changed_object
            base_object_force_save = True
        else:
            # The model in the database contains the most recent data already,
            # or we're not ready to approve the changes stored in
            # ModeratedObject.
            obj_class = self.changed_object.__class__
            pk = self.changed_object.pk
            base_object = obj_class._default_manager.get(pk=pk)
            base_object_force_save = False

        if new_status == MODERATION_STATUS_APPROVED:
            # This version is now approved, and will be reverted to if
            # future changes are rejected by a moderator.
            self.state = MODERATION_READY_STATE

        self.status = new_status
        self.on = datetime.datetime.now()
        self.by = by
        self.reason = reason
        self.save()

        if self.moderator.visibility_column:
            old_visible = getattr(base_object,
                                  self.moderator.visibility_column)

            if new_status == MODERATION_STATUS_APPROVED:
                new_visible = True
            elif new_status == MODERATION_STATUS_REJECTED:
                new_visible = False
            else:  # MODERATION_STATUS_PENDING
                new_visible = self.moderator.visible_until_rejected

            if new_visible != old_visible:
                setattr(base_object, self.moderator.visibility_column,
                        new_visible)
                base_object_force_save = True

        if base_object_force_save:
            # avoid triggering pre/post_save_handler
            with transaction.atomic(using=None, savepoint=False):
                base_object.save_base(raw=True)
                # The _save_parents call is required for models with an
                # inherited visibility_column.
                base_object._save_parents(base_object.__class__, None, None)

        if self.changed_by:
            self.moderator.inform_user(self.content_object, self.changed_by)

    def has_object_been_changed(self, original_obj, only_excluded=False):
        excludes = includes = []
        if only_excluded:
            includes = self.moderator.fields_exclude
        else:
            excludes = self.moderator.fields_exclude

        changes = get_changes_between_models(original_obj,
                                             self.changed_object,
                                             excludes,
                                             includes)

        for change in changes:
            left_change, right_change = changes[change].change
            if left_change != right_change:
                return True

        return False

    def approve(self, by=None, reason=None):
        self._send_signals_and_moderate(MODERATION_STATUS_APPROVED, by, reason)

    def reject(self, by=None, reason=None):
        self._send_signals_and_moderate(MODERATION_STATUS_REJECTED, by, reason)



from .unit.testregister import *
from .unit.testmanagers import *
from .unit.testmodels import *
from .unit.testadmin import *
from .unit.testdiff import *
from .unit.testforms import *
from .unit.testmoderator import *
from .regression import *
from .acceptance.testexclude import *
from .acceptance.testauto_discover import *



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Book',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=20)),
                ('author', models.CharField(max_length=20)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='CustomUser',
            fields=[
                ('user_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('date_of_birth', models.DateField(null=True, blank=True)),
                ('height', models.FloatField(null=True, blank=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
            },
            bases=('auth.user',),
        ),
        migrations.CreateModel(
            name='CustomModel',
            fields=[
                ('name', models.CharField(max_length=20)),
            ],
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MyTestModel',
            fields=[
                ('name', models.CharField(max_length=20)),
            ],
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelClass',
            fields=[
            ],
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWIthDateField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField(auto_now=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('image', models.ImageField(upload_to='tmp')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithModeratedFields',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('moderated', models.CharField(max_length=20)),
                ('also_moderated', models.CharField(max_length=20)),
                ('unmoderated', models.CharField(max_length=20)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithMultipleManagers',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('gender', models.SmallIntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithSlugField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithSlugField2',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithVisibilityField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('test', models.CharField(max_length=20)),
                ('is_public', models.BooleanField(default=False)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ModelWithWrongVisibilityField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('test', models.CharField(max_length=20)),
                ('is_public', models.IntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('description', models.TextField()),
                ('url', models.URLField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SuperUserProfile',
            fields=[
                ('userprofile_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='tests.UserProfile')),
                ('super_power', models.TextField()),
            ],
            options={
            },
            bases=('tests.userprofile',),
        ),
        migrations.CreateModel(
            name='UserProfileWithCustomUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('description', models.TextField()),
                ('url', models.URLField()),
                ('user', models.ForeignKey(to='tests.CustomUser')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='user',
            field=models.ForeignKey(related_name='user_profile_set', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.CreateModel(
            name='ProxyProfile',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('tests.userprofile',),
        ),
    ]



from __future__ import unicode_literals
from django.test.testcases import TestCase
from django.contrib.auth.models import User
from tests.models import UserProfile, \
    ModelWithSlugField2, ModelWithVisibilityField
from moderation.managers import ModerationObjectsManager
from django.db.models.manager import Manager
from moderation.models import ModeratedObject
from django.core.exceptions import MultipleObjectsReturned
from moderation.moderator import GenericModerator
from tests.utils import setup_moderation, teardown_moderation


class ModerationObjectsManagerTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):

        self.user = User.objects.get(username='moderator')
        self.profile = UserProfile.objects.get(user__username='moderator')

        class UserProfileModerator(GenericModerator):
            visibility_column = 'is_public'

        self.moderation = setup_moderation(
            [
                UserProfile,
                (ModelWithVisibilityField, UserProfileModerator)])

    def tearDown(self):
        teardown_moderation()

    def test_moderation_objects_manager(self):
        ManagerClass = ModerationObjectsManager()(Manager)

        self.assertEqual(
            str(ManagerClass.__bases__),
            "(<class 'moderation.managers.ModerationObjectsManager'>"
            ", <class 'django.db.models.manager.Manager'>)")

    def test_filter_moderated_objects_returns_empty_queryset(self):
        """Test filter_moderated_objects returns empty queryset
        for object that has moderated object"""

        ManagerClass = ModerationObjectsManager()(Manager)
        manager = ManagerClass()
        manager.model = UserProfile

        query_set = UserProfile._default_manager.all()
        moderated_object = ModeratedObject(content_object=self.profile)
        moderated_object.save()

        self.assertEqual(str(manager.filter_moderated_objects(query_set)),
                         "[]")

    def test_filter_moderated_objects_returns_object(self):
        """Test if filter_moderated_objects return object when object 
        doesn't have moderated object or deserialized object is <> object"""
        moderated_object = ModeratedObject(content_object=self.profile)
        moderated_object.save()
        moderated_object.approve()

        self.profile.description = "New"
        self.profile.save()

        self.assertEqual(str(UserProfile.objects.all()),
                         '[<UserProfile: moderator - http://www.google.com>]')

    def test_exclude_objs_by_visibility_col(self):
        ManagerClass = ModerationObjectsManager()(Manager)
        manager = ManagerClass()
        manager.model = ModelWithVisibilityField

        ModelWithVisibilityField(test='test 1').save()
        ModelWithVisibilityField(test='test 2', is_public=True).save()

        query_set = ModelWithVisibilityField.objects.all()

        query_set = manager.exclude_objs_by_visibility_col(query_set)

        self.assertEqual(
            str(query_set),
            "[<ModelWithVisibilityField: test 2 - is public True>]")


class ModeratedObjectManagerTestCase(TestCase):
    fixtures = ['test_users.json']

    def setUp(self):
        self.moderation = setup_moderation([UserProfile, ModelWithSlugField2])

        self.user = User.objects.get(username='admin')

    def tearDown(self):
        teardown_moderation()

    def test_objects_with_same_object_id(self):
        model1 = ModelWithSlugField2(slug='test')
        model1.save()

        model2 = UserProfile(description='Profile for new user',
                             url='http://www.yahoo.com',
                             user=User.objects.get(username='user1'))

        model2.save()

        self.assertRaises(MultipleObjectsReturned,
                          ModeratedObject.objects.get,
                          object_pk=model2.pk)

        moderated_obj1 = ModeratedObject.objects.get_for_instance(model1)
        moderated_obj2 = ModeratedObject.objects.get_for_instance(model2)

        self.assertEqual(repr(moderated_obj1),
                         "<ModeratedObject: ModelWithSlugField2 object>")
        self.assertEqual(repr(moderated_obj2),
                         '<ModeratedObject: user1 - http://www.yahoo.com>')

    def test_instance_with_many_moderations(self):
        # Register a moderator that keeps the history
        # un-register UserProfile
        from moderation import moderation

        class KeepHistoryModerator(GenericModerator):
            keep_history = True

        moderation.unregister(UserProfile)
        moderation.register(UserProfile, KeepHistoryModerator)

        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=self.user)
        profile.save()

        # There should only be one ModeratedObject
        self.assertEqual(1, ModeratedObject.objects.count())

        profile.url = 'http://www.google.com'
        profile.save()

        # Should now be two
        self.assertEqual(2, ModeratedObject.objects.count())

        # Getting for instance should return the one with the google url
        moderated_object = ModeratedObject.objects.get_for_instance(profile)
        self.assertEqual(profile.url, moderated_object.changed_object.url)

        # Get the first object, and does it have the yahoo address
        moderated_object_pk1 = ModeratedObject.objects.get(pk=1)
        self.assertEqual('http://www.yahoo.com',
                         moderated_object_pk1.changed_object.url)



from __future__ import unicode_literals
from django.contrib.auth.models import User
from django.core import management
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.manager import Manager
from django.test.testcases import TestCase
from moderation.register import ModerationManager, RegistrationError
from moderation.moderator import GenericModerator
from moderation.managers import ModerationObjectsManager
from moderation.models import ModeratedObject
from moderation.constants import (MODERATION_STATUS_REJECTED,
                                  MODERATION_STATUS_APPROVED,
                                  MODERATION_STATUS_PENDING)
from moderation.signals import pre_moderation, post_moderation
from tests.models import UserProfile, \
    ModelWithSlugField, ModelWithSlugField2, ModelWithMultipleManagers, \
    CustomModel
from tests.utils import setup_moderation
from tests.utils import teardown_moderation
from moderation.helpers import import_moderator
from tests.models import Book
# reload is builtin in Python 2.x. Needs to  be imported for Py3k
try:
    from importlib import reload
except ImportError:
    try:
        # Python 3.2
        from imp import reload
    except:
        pass

from django.db import IntegrityError, transaction


class MyModelModerator(GenericModerator):
    pass


class RegistrationTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        self.moderation = setup_moderation([UserProfile])
        self.user = User.objects.get(username='moderator')

    def tearDown(self):
        teardown_moderation()

    def test_get_moderator(self):
        moderator = self.moderation.get_moderator(UserProfile)

        self.assertTrue(isinstance(moderator, GenericModerator))

    def test_get_of_new_object_should_raise_exception(self):
        """Tests if after register of model class with moderation, 
           when new object is created and getting of object 
           raise ObjectDoesNotExist"""

        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        self.assertRaises(ObjectDoesNotExist, UserProfile.objects.get,
                          pk=profile.pk)

    def test_creation_of_moderated_object(self):
        """
        Test if after create of new object moderated object should be created
        """
        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        moderated_object = ModeratedObject.objects.get_for_instance(profile)

        self.assertEqual(str(moderated_object),
                         "user1 - http://www.yahoo.com")

    def test_get_of_existing_object_should_return_old_version_of_object(self):
        """Tests if after register of model class with moderation, 
            when existing object is saved, when getting of object returns 
            old version of object"""
        profile = UserProfile.objects.get(user__username='moderator')
        moderated_object = ModeratedObject(content_object=profile)
        moderated_object.save()
        moderated_object.approve(by=self.user)

        profile.description = "New description"
        profile.save()

        old_profile = UserProfile.objects.get(pk=profile.pk)

        self.assertEqual(old_profile.description, 'Old description')

    def test_can_use_object_without_moderated_object(self):
        """
        For backwards compatibility, when django-moderation is added to an
        existing project, records with no ModeratedObject should be visible
        as before.
        """

        profile = UserProfile.objects.get(user__username='moderator')
        # Pretend that it was created before django-moderation was installed,
        # by deleting the ModeratedObject.
        ModeratedObject.objects.filter(object_pk=profile.pk).delete()

        # We should be able to load it
        profile = UserProfile.objects.get(user__username='moderator')
        # And save changes to it
        profile.description = "New description"
        profile.save()
        # And now it should be invisible, because it's pending
        self.assertEqual(
            [], list(UserProfile.objects.all()),
            "The previously unmoderated object should now be invisible, "
            "because it has never been accepted.")

    def test_register(self):
        """Tests if after creation of new model instance new 
        moderation object is created"""
        UserProfile(description='Profile for new user',
                    url='http://www.yahoo.com',
                    user=User.objects.get(username='user1')).save()

        self.assertEqual(ModeratedObject.objects.all().count(), 1,
                         "New moderation object was not created"
                         " after creation of new model instance "
                         "from model class that is registered with moderation")

    def test_exception_is_raised_when_class_is_registered(self):
        self.assertRaises(RegistrationError, self.moderation.register,
                          UserProfile)

    def test_custom_moderator_should_be_registered_with_moderation(self):
        self.moderation.register(CustomModel, MyModelModerator)
        moderator_instance = self.moderation._registered_models[CustomModel]

        self.assertTrue(isinstance(moderator_instance, MyModelModerator))


class AutoDiscoverTestCase(TestCase):
    urls = 'tests.urls.auto_register'

    def setUp(self):
        self.moderation = setup_moderation()

    def tearDown(self):
        teardown_moderation()

    def test_models_should_be_registered_if_moderator_in_module(self):
        module = import_moderator('tests')

        try:  # force module reload
            reload(module)
        except:
            pass

        self.assertTrue(Book in self.moderation._registered_models)
        self.assertEqual(module.__name__,
                         'tests.moderator')


class RegisterMultipleManagersTestCase(TestCase):

    def setUp(self):
        self.moderation = ModerationManager()

        class ModelWithMultipleManagersModerator(GenericModerator):
            manager_names = ['objects', 'men', 'women']

        setup_moderation([(ModelWithMultipleManagers,
                           ModelWithMultipleManagersModerator)])

    def tearDown(self):
        teardown_moderation()

    def test_multiple_managers(self):
        obj = ModelWithMultipleManagers(gender=0)
        obj.save()

        obj2 = ModelWithMultipleManagers(gender=1)
        obj2.save()

        men = ModelWithMultipleManagers.men.all()
        women = ModelWithMultipleManagers.women.all()

        self.assertEqual(men.count(), 0)
        self.assertEqual(women.count(), 0)


class IntegrityErrorTestCase(TestCase):

    def setUp(self):
        self.moderation = setup_moderation([ModelWithSlugField])

    def tearDown(self):
        teardown_moderation()

    def test_raise_integrity_error_model_registered_with_moderation(self):
        m1 = ModelWithSlugField(slug='test')
        m1.save()

        self.assertRaises(ObjectDoesNotExist, ModelWithSlugField.objects.get,
                          slug='test')

        m2 = ModelWithSlugField(slug='test')

        if hasattr(transaction, 'atomic'):
            with transaction.atomic():
                self.assertRaises(IntegrityError, m2.save)
        else:
            self.assertRaises(IntegrityError, m2.save)

        self.assertEqual(ModeratedObject.objects.all().count(), 1)

    def test_raise_integrity_error_model_not_registered_with_moderation(self):
        m1 = ModelWithSlugField2(slug='test')
        m1.save()

        m1 = ModelWithSlugField2.objects.get(slug='test')

        m2 = ModelWithSlugField2(slug='test')
        if hasattr(transaction, 'atomic'):
            with transaction.atomic():
                self.assertRaises(IntegrityError, m2.save)
        else:
            self.assertRaises(IntegrityError, m2.save)

        self.assertEqual(ModeratedObject.objects.all().count(), 0)


class IntegrityErrorRegressionTestCase(TestCase):

    def setUp(self):
        self.moderation = ModerationManager()
        self.moderation.register(ModelWithSlugField)
        self.filter_moderated_objects = ModelWithSlugField.objects.\
            filter_moderated_objects

        def filter_moderated_objects(query_set):
            exclude_pks = []
            for obj in query_set:
                try:
                    if obj.moderated_object.status\
                       in [MODERATION_STATUS_PENDING,
                           MODERATION_STATUS_REJECTED]\
                       and obj.__dict__ == \
                       obj.moderated_object.changed_object.__dict__:
                        exclude_pks.append(object.pk)
                except ObjectDoesNotExist:
                    pass

            return query_set.exclude(pk__in=exclude_pks)

        setattr(ModelWithSlugField.objects,
                'filter_moderated_objects',
                filter_moderated_objects)

    def tearDown(self):
        self.moderation.unregister(ModelWithSlugField)

    def test_old_version_of_filter_moderated_objects_method(self):
        m1 = ModelWithSlugField(slug='test')
        m1.save()

        m2 = ModelWithSlugField(slug='test')
        if hasattr(transaction, 'atomic'):
            with transaction.atomic():
                self.assertRaises(IntegrityError, m2.save)
        else:
            self.assertRaises(IntegrityError, m2.save)

        self.assertEqual(ModeratedObject.objects.all().count(), 1)


class ModerationManagerTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        self.moderation = setup_moderation()
        self.user = User.objects.get(username='moderator')

    def tearDown(self):
        teardown_moderation()

    def test_unregister(self):
        """Tests if model class is successfully unregistered from moderation"""
        from django.db.models import signals

        old_pre_save_receivers = [r for r in signals.pre_save.receivers]
        old_post_save_receivers = [r for r in signals.post_save.receivers]

        signals.pre_save.receivers = []
        signals.post_save.receivers = []
        self.moderation.register(UserProfile)

        self.assertNotEqual(signals.pre_save.receivers, [])
        self.assertNotEqual(signals.post_save.receivers, [])

        UserProfile(description='Profile for new user',
                    url='http://www.yahoo.com',
                    user=User.objects.get(username='user1')).save()

        self.moderation.unregister(UserProfile)

        self.assertEqual(signals.pre_save.receivers, [])
        self.assertEqual(signals.post_save.receivers, [])

        self.assertEqual(UserProfile.objects.__class__, Manager)
        self.assertEqual(hasattr(UserProfile, 'moderated_object'), False)

        signals.pre_save.receivers = old_pre_save_receivers
        signals.post_save.receivers = old_post_save_receivers

        UserProfile.objects.get(user__username='user1')

        User.objects.get(username='moderator')
        management.call_command('loaddata', 'test_moderation.json',
                                verbosity=0)

    def test_moderation_manager(self):
        moderation = ModerationManager()

        self.assertEqual(moderation._registered_models, {})

    def test_save_new_instance_after_add_and_remove_fields_from_class(self):
        """Test if after removing moderation from model class new 
        instance of model can be created"""

        class CustomManager(Manager):
            pass

        moderator = GenericModerator(UserProfile)
        self.moderation._and_fields_to_model_class(moderator)

        self.moderation._remove_fields(moderator)

        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        up = UserProfile._default_manager.filter(url='http://www.yahoo.com')
        self.assertEqual(up.count(), 1)

    def test_and_fields_to_model_class(self):

        class CustomManager(Manager):
            pass

        moderator = GenericModerator(UserProfile)
        self.moderation._and_fields_to_model_class(moderator)

        manager = ModerationObjectsManager()(CustomManager)()

        self.assertEqual(repr(UserProfile.objects.__class__),
                         repr(manager.__class__))
        self.assertEqual(hasattr(UserProfile, 'moderated_object'), True)

        # clean up
        self.moderation._remove_fields(moderator)

    def test_get_or_create_moderated_object_exist(self):
        self.moderation.register(UserProfile)
        profile = UserProfile.objects.get(user__username='moderator')

        moderator = self.moderation.get_moderator(UserProfile)

        ModeratedObject(content_object=profile).save()

        profile.description = "New description"

        unchanged_obj = self.moderation._get_unchanged_object(profile)
        object = self.moderation._get_or_create_moderated_object(profile,
                                                                 unchanged_obj,
                                                                 moderator)

        self.assertNotEqual(object.pk, None)
        self.assertEqual(object.changed_object.description,
                         'Old description')

        self.moderation.unregister(UserProfile)

    def test_get_or_create_moderated_object_does_not_exist(self):
        profile = UserProfile.objects.get(user__username='moderator')
        profile.description = "New description"

        self.moderation.register(UserProfile)
        moderator = self.moderation.get_moderator(UserProfile)

        unchanged_obj = self.moderation._get_unchanged_object(profile)

        object = self.moderation._get_or_create_moderated_object(profile,
                                                                 unchanged_obj,
                                                                 moderator)

        self.assertEqual(object.pk, None)
        self.assertEqual(object.changed_object.description,
                         'Old description')

        self.moderation.unregister(UserProfile)

    def test_get_or_create_moderated_object_keep_history(self):
        profile = UserProfile.objects.get(user__username='moderator')
        profile.description = "New description"

        self.moderation.register(UserProfile)
        moderator = self.moderation.get_moderator(UserProfile)
        moderator.keep_history = True

        unchanged_obj = self.moderation._get_unchanged_object(profile)

        moderated_object = self.moderation._get_or_create_moderated_object(
            profile, unchanged_obj, moderator)
        self.assertEqual(moderated_object.pk, None)
        self.assertEqual(moderated_object.changed_object.description,
                         'Old description')

        moderated_object.save()

        # moderated_object should have a pk now, and since it's the first one
        # it should be 1
        self.assertEqual(1, moderated_object.pk)

        # If we call it again, we should get a new moderated_object, evidenced
        # by having no pk

        moderated_object_2 = self.moderation._get_or_create_moderated_object(
            profile, unchanged_obj, moderator)

        self.assertEqual(moderated_object_2.pk, None)
        self.assertEqual(moderated_object_2.changed_object.description,
                         'Old description')

    def test_get_unchanged_object(self):
        profile = UserProfile.objects.get(user__username='moderator')
        profile.description = "New description"

        object = self.moderation._get_unchanged_object(profile)

        self.assertEqual(object.description,
                         'Old description')


class LoadingFixturesTestCase(TestCase):
    fixtures = ['test_users.json']

    def setUp(self):
        self.new_moderation = setup_moderation([UserProfile])
        self.user = User.objects.get(username='moderator')

    def tearDown(self):
        teardown_moderation()

    def test_loading_fixture_for_moderated_model(self):
        management.call_command('loaddata', 'test_moderation.json',
                                verbosity=0)

        self.assertEqual(UserProfile.objects.all().count(), 1)

    def test_loading_objs_from_fixture_should_not_create_moderated_obj(self):
        management.call_command('loaddata', 'test_moderation.json',
                                verbosity=0)

        profile = UserProfile.objects.get(user__username='moderator')

        self.assertRaises(ObjectDoesNotExist,
                          ModeratedObject.objects.get, object_pk=profile.pk)

    def test_moderated_object_is_created_when_not_loaded_from_fixture(self):
        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        moderated_objs = ModeratedObject.objects.filter(object_pk=profile.pk)
        self.assertEqual(moderated_objs.count(), 1)


class ModerationSignalsTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):

        class UserProfileModerator(GenericModerator):

            notify_moderator = False

        self.moderation = setup_moderation(
            [(UserProfile, UserProfileModerator)])

        self.moderation._disconnect_signals(UserProfile)

        self.user = User.objects.get(username='moderator')
        self.profile = UserProfile.objects.get(user__username='moderator')

    def tearDown(self):
        teardown_moderation()

    def test_send_pre_moderation_signal(self):
        """check if custom_approve_handler function was called when """
        """moderation_approve signal was send"""

        def custom_pre_moderation_handler(sender, instance, status, **kwargs):
            # do some stuff with approved instance
            instance.description = 'Change description'
            instance.save()

        pre_moderation.connect(custom_pre_moderation_handler,
                               sender=UserProfile)

        pre_moderation.send(sender=UserProfile, instance=self.profile,
                            status=MODERATION_STATUS_APPROVED)

        self.assertEqual(self.profile.description, 'Change description')

    def test_send_post_moderation_signal(self):
        """check if custom_approve_handler function was called when """
        """moderation_approve signal was send"""

        def custom_post_moderation_handler(sender, instance, status, **kwargs):
            # do some stuff with approved instance
            instance.description = 'Change description'
            instance.save()

        post_moderation.connect(custom_post_moderation_handler,
                                sender=UserProfile)

        post_moderation.send(sender=UserProfile, instance=self.profile,
                             status=MODERATION_STATUS_APPROVED)

        self.assertEqual(self.profile.description, 'Change description')

    def test_connect_and_disconnect_signals(self):
        from django.db.models import signals

        old_pre_save_receivers = [r for r in signals.pre_save.receivers]
        old_post_save_receivers = [r for r in signals.post_save.receivers]

        signals.pre_save.receivers = []
        signals.post_save.receivers = []

        self.moderation._connect_signals(UserProfile)

        self.assertNotEqual(signals.pre_save.receivers, [])
        self.assertNotEqual(signals.post_save.receivers, [])

        self.moderation._disconnect_signals(UserProfile)

        self.assertEqual(signals.pre_save.receivers, [])
        self.assertEqual(signals.post_save.receivers, [])

        signals.pre_save.receivers = old_pre_save_receivers
        signals.post_save.receivers = old_post_save_receivers

    def test_after_disconnecting_signals_moderation_object(self):
        self.moderation._connect_signals(UserProfile)
        self.moderation._disconnect_signals(UserProfile)

        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        self.assertRaises(ObjectDoesNotExist, ModeratedObject.objects.get,
                          object_pk=profile.pk)

    def test_post_save_handler_for_existing_object(self):
        from django.db.models import signals

        signals.pre_save.connect(self.moderation.pre_save_handler,
                                 sender=UserProfile)
        signals.post_save.connect(self.moderation.post_save_handler,
                                  sender=UserProfile)
        profile = UserProfile.objects.get(user__username='moderator')
        moderated_object = ModeratedObject(content_object=profile)
        moderated_object.save()
        moderated_object.approve(by=self.user)

        profile.description = 'New description of user profile'
        profile.save()

        moderated_object = ModeratedObject.objects.get_for_instance(profile)

        original_object = moderated_object.changed_object
        self.assertEqual(original_object.description,
                         'New description of user profile')
        self.assertEqual(UserProfile.objects.get(pk=profile.pk).description,
                         'Old description')

        signals.pre_save.disconnect(self.moderation.pre_save_handler,
                                    UserProfile)
        signals.post_save.disconnect(self.moderation.post_save_handler,
                                     UserProfile)

    def test_pre_save_handler_for_existing_object(self):
        from django.db.models import signals

        signals.pre_save.connect(self.moderation.pre_save_handler,
                                 sender=UserProfile)

        profile = UserProfile.objects.get(user__username='moderator')

        profile.description = 'New description of user profile'
        profile.save()

        moderated_object = ModeratedObject.objects.get_for_instance(profile)

        original_object = moderated_object.changed_object
        content_object = moderated_object.content_object

        self.assertEqual(original_object.description,
                         'Old description')
        self.assertEqual(content_object.description,
                         'New description of user profile')

        signals.pre_save.disconnect(self.moderation.pre_save_handler,
                                    UserProfile)

    def test_post_save_handler_for_new_object(self):
        from django.db.models import signals

        signals.pre_save.connect(self.moderation.pre_save_handler,
                                 sender=UserProfile)
        signals.post_save.connect(self.moderation.post_save_handler,
                                  sender=UserProfile)
        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        moderated_object = ModeratedObject.objects.get_for_instance(profile)

        self.assertEqual(moderated_object.content_object, profile)

        signals.pre_save.disconnect(self.moderation.pre_save_handler,
                                    UserProfile)
        signals.post_save.disconnect(self.moderation.post_save_handler,
                                     UserProfile)

    def test_save_handler_keep_history(self):
        # de-register current Moderator and replace it with one that
        # has keep_history set to True
        from moderation import moderation

        class KeepHistoryModerator(GenericModerator):
            keep_history = True
            notify_moderator = False

        moderation.unregister(UserProfile)
        moderation.register(UserProfile, KeepHistoryModerator)

        from django.db.models import signals

        signals.pre_save.connect(self.moderation.pre_save_handler,
                                 sender=UserProfile)
        signals.post_save.connect(self.moderation.post_save_handler,
                                  sender=UserProfile)
        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        moderated_object = ModeratedObject.objects.get_for_instance(profile)

        self.assertEqual(moderated_object.content_object, profile)

        # Now update it and make sure it gets the right history object...
        profile.url = 'http://www.google.com'
        profile.save()

        moderated_object = ModeratedObject.objects.get_for_instance(profile)
        self.assertEqual(moderated_object.content_object, profile)

        # There should only be two moderated objects
        self.assertEqual(2, ModeratedObject.objects.count())

        # Approve the change
        moderated_object.approve(by=self.user,
                                 reason='Testing post save handlers')

        # There should *still* only be two moderated objects
        self.assertEqual(2, ModeratedObject.objects.count())

        signals.pre_save.disconnect(self.moderation.pre_save_handler,
                                    UserProfile)
        signals.post_save.disconnect(self.moderation.post_save_handler,
                                     UserProfile)

        self.moderation = False

    def test_pre_save_handler_for_new_object(self):
        from django.db.models import signals

        signals.pre_save.connect(self.moderation.pre_save_handler,
                                 sender=UserProfile)
        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        self.assertRaises(ObjectDoesNotExist,
                          ModeratedObject.objects.get_for_instance,
                          profile)

        signals.pre_save.disconnect(self.moderation.pre_save_handler,
                                    UserProfile)



from __future__ import unicode_literals

import django.dispatch

pre_moderation = django.dispatch.Signal(providing_args=["instance", "status"])

post_moderation = django.dispatch.Signal(providing_args=["instance", "status"])

pre_many_moderation = django.dispatch.Signal(providing_args=["queryset", "status", "by", "reason"])

post_many_moderation = django.dispatch.Signal(providing_args=["queryset", "status", "by", "reason"])



from __future__ import unicode_literals
from django.db.models.fields.files import ImageFieldFile
from django.forms import CharField
from django.contrib.auth.models import User
from django.test.testcases import TestCase

from tests.models import UserProfile, ModelWithImage
from moderation.forms import BaseModeratedObjectForm
from tests.utils import setup_moderation, teardown_moderation
from moderation.utils import django_17


class FormsTestCase(TestCase):
    fixtures = ['test_users.json']

    def setUp(self):
        self.user = User.objects.get(username='moderator')

        class ModeratedObjectForm(BaseModeratedObjectForm):
            extra = CharField(required=False)

            class Meta:
                model = UserProfile
                if django_17():
                    fields = '__all__'

        self.ModeratedObjectForm = ModeratedObjectForm
        self.moderation = setup_moderation([UserProfile, ModelWithImage])

    def tearDown(self):
        teardown_moderation()

    def test_create_form_class(self):
        form = self.ModeratedObjectForm()
        self.assertEqual(form._meta.model.__name__, 'UserProfile')

    def test_if_form_is_initialized_new_object(self):
        profile = UserProfile(description="New description",
                              url='http://test.com',
                              user=self.user)
        profile.save()

        form = self.ModeratedObjectForm(instance=profile)
        self.assertEqual(form.initial['description'], 'New description')

    def test_if_form_is_initialized_existing_object(self):
        profile = UserProfile(description="old description",
                              url='http://test.com',
                              user=self.user)
        profile.save()

        profile.moderated_object.approve(by=self.user)

        profile.description = "Changed description"
        profile.save()

        form = self.ModeratedObjectForm(instance=profile)

        profile = UserProfile.objects.get(id=1)

        self.assertEqual(profile.description, "old description")
        self.assertEqual(form.initial['description'], 'Changed description')

    def test_if_form_has_image_field_instance_of_image_field_file(self):
        object = ModelWithImage(image='my_image.jpg')
        object.save()

        object = ModelWithImage.unmoderated_objects.get(id=1)
        form = self.ModeratedObjectForm(instance=object)
        self.assertTrue(isinstance(form.initial['image'], ImageFieldFile),
                        'image in form.initial is instance of ImageField File')

    def test_form_when_obj_has_no_moderated_obj(self):
        self.moderation.unregister(UserProfile)
        profile = UserProfile(description="old description",
                              url='http://test.com',
                              user=self.user)
        profile.save()
        self.moderation.register(UserProfile)

        form = self.ModeratedObjectForm(instance=profile)

        self.assertEqual(form.initial['description'], 'old description')

    def test_if_form_is_initialized_new_object_with_initial(self):
        profile = UserProfile(description="New description",
                              url='http://test.com',
                              user=self.user)
        profile.save()

        form = self.ModeratedObjectForm(initial={'extra': 'value'},
                                        instance=profile)

        self.assertEqual(form.initial['description'], 'New description')
        self.assertEqual(form.initial['extra'], 'value')



from __future__ import unicode_literals

import sys

from django.test.testcases import TestCase

from tests.utils import setup_moderation, teardown_moderation
from tests.models import Book
from moderation.helpers import auto_discover
from moderation.utils import django_17


class AutoDiscoverAcceptanceTestCase(TestCase):
    '''
    As a developer I want to have a way auto discover all apps that have module
    moderator and register it with moderation.
    '''
    urls = 'tests.urls.auto_discover'

    def setUp(self):
        setup_moderation()

    def tearDown(self):
        teardown_moderation()

    def test_all_app_containing_moderator_module_should_be_registered(self):
        auto_discover()

        if django_17():
            self.assertTrue("tests.moderator" in sys.modules.keys())
        else:

            from moderation import moderation

            self.assertTrue(Book in moderation._registered_models)



from setuptools import setup, find_packages
import os
import sys

version = '0.4.0'

tests_require = [
    'django>=1.7,<1.10',
    'django-webtest>=1.5.7-web',
    'webtest>=2.0,<2.1',
    'mock',
    'pillow',
]

# ipython>2 is only supported on Python 2.7+
if sys.hexversion < 0x02070000:
    tests_require = ['ipython>=0.10,<2'] + tests_require

if sys.hexversion >= 0x03000000:
    tests_require = ['unittest2py3k'] + tests_require
else:
    tests_require = ['unittest2'] + tests_require

setup(
    name='django-moderation',
    version=version,
    description="Generic Django objects moderation application",
    long_description=open("README.rst").read() + "\n" +
                     open(os.path.join("docs", "history.rst")).read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    keywords='django moderation models',
    author='Dominik Szopa',
    author_email='dszopa@gmail.com',
    url='http://github.com/dominno/django-moderation',
    license='BSD',
    packages=find_packages('.', exclude=('tests', 'example_project')),
    include_package_data=True,
    tests_require=tests_require,
    test_suite='runtests.runtests',
    install_requires=[
        'django>=1.7,<1.10',
        'django-model-utils',
    ],
    zip_safe=False,
)



from __future__ import unicode_literals

import django
from django.test.testcases import TestCase
from django.contrib.auth.models import User

from moderation.helpers import automoderate
from moderation.constants import MODERATION_STATUS_APPROVED
from moderation.moderator import GenericModerator

from tests.models import UserProfile, ModelWithVisibilityField
from tests.utils import setup_moderation, teardown_moderation

django_version = django.get_version()[:3]


class CSRFMiddlewareTestCase(TestCase):
    fixtures = ['test_users.json']

    def setUp(self):
        setup_moderation([UserProfile])

    def tearDown(self):
        teardown_moderation()

    def test_csrf_token(self):
        profile = UserProfile(description='Profile for new user',
                              url='http://www.yahoo.com',
                              user=User.objects.get(username='user1'))

        profile.save()

        self.client.login(username='admin', password='aaaa')

        url = profile.moderated_object.get_admin_moderate_url()

        if django_version == '1.1':
            from django.contrib.csrf.middleware import _make_token

            csrf_token = _make_token(self.client.session.session_key)
            post_data = {'approve': 'Approve',
                         'csrfmiddlewaretoken': csrf_token}
        else:
            post_data = {'approve': 'Approve'}

        response = self.client.post(url, post_data)

        self.assertEqual(response.status_code, 302)

        profile = UserProfile.objects.get(pk=profile.pk)

        self.assertEqual(profile.moderated_object.status,
                         MODERATION_STATUS_APPROVED)


class AutomoderationRuntimeErrorRegressionTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        setup_moderation([UserProfile])

        self.user = User.objects.get(username='admin')

    def tearDown(self):
        teardown_moderation()

    def test_RuntimeError(self):
        from moderation.helpers import automoderate

        profile = UserProfile.objects.get(user__username='moderator')
        profile.description = 'Change description'
        profile.save()
        profile.moderated_object.changed_by = self.user
        profile.moderated_object.save()
        automoderate(profile, self.user)
        profile.moderated_object.save()


class BypassOverwritesUpdatedObjectRegressionTestCase(TestCase):
    fixtures = ['test_users.json', 'test_moderation.json']

    def setUp(self):
        class BypassModerator(GenericModerator):
            visibility_column = 'is_public'
            bypass_moderation_after_approval = True

        setup_moderation([(ModelWithVisibilityField, BypassModerator)])
        self.user = User.objects.get(username='admin')

    def tearDown(self):
        teardown_moderation()

    def test_can_update_objects_with_bypass_enabled(self):
        obj = ModelWithVisibilityField.objects.create(test='initial')
        obj.save()

        # It's never been approved before, so it's now invisible
        self.assertEqual(
            [], list(ModelWithVisibilityField.objects.all()),
            "The ModelWithVisibilityField has never been approved and is now "
            "pending, so it should be hidden")
        # So approve it
        obj.moderated_object.approve(by=self.user, reason='test')
        # Now it should be visible, with the new description
        obj = ModelWithVisibilityField.objects.get()
        self.assertEqual('initial', obj.test)

        # Now change it again. Because bypass_moderation_after_approval is
        # True, it should still be visible and we shouldn't need to approve it
        # again.
        obj.test = 'modified'
        obj.save()
        obj = ModelWithVisibilityField.objects.get()
        self.assertEqual('modified', obj.test)

        # Admin does this after saving an object. Check that it doesn't undo
        # our changes.
        automoderate(obj, self.user)
        obj = ModelWithVisibilityField.objects.get()
        self.assertEqual('modified', obj.test)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-24 14:00
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('moderation', '0002_auto_20150515_1057'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='moderatedobject',
            options={
                'ordering': [
                    'status',
                    'created'
                ],
                'verbose_name': 'Moderated Object',
                'verbose_name_plural': 'Moderated Objects'
            },
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='date_created',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='moderation_date',
            new_name='on',
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='moderation_reason',
            new_name='reason',
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='moderation_state',
            new_name='state',
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='date_updated',
            new_name='updated',
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='moderated_by',
            new_name='by',
        ),
        migrations.RenameField(
            model_name='moderatedobject',
            old_name='moderation_status',
            new_name='status',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import difflib
import re
import sys

from django.db.models import fields
try:
    from django.db.models.fields.related import ForeignObject
except ImportError:
    from django.db.models.fields.related import RelatedField as ForeignObject
from django.utils.html import escape


class BaseChange(object):

    def __repr__(self):
        value1, value2 = self.change
        return 'Change object: %s - %s' % (value1, value2)

    def __init__(self, verbose_name, field, change):
        self.verbose_name = verbose_name
        self.field = field
        self.change = change

    def render_diff(self, template, context):
        from django.template.loader import render_to_string

        return render_to_string(template, context)


class TextChange(BaseChange):

    @property
    def diff(self):
        value1, value2 = escape(self.change[0]), escape(self.change[1])
        if value1 == value2:
            return value1

        return self.render_diff(
            'moderation/html_diff.html',
            {'diff_operations': get_diff_operations(*self.change)})


class ImageChange(BaseChange):

    @property
    def diff(self):
        left_image, right_image = self.change
        return self.render_diff(
            'moderation/image_diff.html',
            {'left_image': left_image, 'right_image': right_image})


def get_change(model1, model2, field, resolve_foreignkeys=False):
    try:
        value1 = getattr(model1, "get_%s_display" % field.name)()
        value2 = getattr(model2, "get_%s_display" % field.name)()
    except AttributeError:
        if isinstance(field, ForeignObject) and resolve_foreignkeys:
            value1 = str(getattr(model1, field.name))
            value2 = str(getattr(model2, field.name))
        else:
            value1 = field.value_from_object(model1)
            value2 = field.value_from_object(model2)

    change = get_change_for_type(
        field.verbose_name,
        (value1, value2),
        field,
    )

    return change


def get_changes_between_models(model1, model2, excludes=[], includes=[],
                               resolve_foreignkeys=False):
    changes = {}

    for field in model1._meta.fields:
        if includes and field.name not in includes:
            continue

        if not (isinstance(field, (fields.AutoField,))):
            if field.name in excludes:
                continue

            name = "{}__{}".format(model1.__class__.__name__.lower(), field.name)

            changes[name] = get_change(model1, model2, field,
                                       resolve_foreignkeys)

    return changes


def get_diff_operations(a, b):
    operations = []
    a_words = re.split('(\W+)', a)
    b_words = re.split('(\W+)', b)
    sequence_matcher = difflib.SequenceMatcher(None, a_words, b_words)
    for opcode in sequence_matcher.get_opcodes():
        operation, start_a, end_a, start_b, end_b = opcode

        deleted = ''.join(a_words[start_a:end_a])
        inserted = ''.join(b_words[start_b:end_b])

        operations.append({'operation': operation,
                           'deleted': deleted,
                           'inserted': inserted})
    return operations


def html_to_list(html):
    pattern = re.compile(r'&.*?;|(?:<[^<]*?>)|'
                         '(?:\w[\w-]*[ ]*)|(?:<[^<]*?>)|'
                         '(?:\s*[,\.\?]*)', re.UNICODE)

    return [''.join(element) for element in
            [_f for _f in pattern.findall(html) if _f]]


def get_change_for_type(verbose_name, change, field):
    if isinstance(field, fields.files.ImageField):
        change = ImageChange(
            "Current %(verbose_name)s / "
            "New %(verbose_name)s" % {'verbose_name': verbose_name},
            field,
            change)
    else:
        value1, value2 = change
        if sys.version < '3':
            if value1 and (type(value1) is str or type(value1) is unicode):  # NOQA
                value1 = value1.encode('utf-8')
            if value2 and (type(value2) is str or type(value2) is unicode):  # NOQA
                value2 = value2.encode('utf-8')

        change = TextChange(
            verbose_name,
            field,
            (str(value1), str(value2)),
        )

    return change



from __future__ import unicode_literals
from django.conf import settings
from django.core.mail import send_mail, send_mass_mail


class BaseMessageBackend(object):

    def send(self, **kwargs):
        raise NotImplementedError


class BaseMultipleMessageBackend(BaseMessageBackend):
    """Used to send mail to multiple users"""


class SyncMessageBackend(BaseMessageBackend):
    """Synchronous backend"""


class AsyncMessageBackend(BaseMessageBackend):
    """Asynchronous backend"""


class EmailMessageBackend(SyncMessageBackend):
    """
    Send the message through an email on the main thread
    """

    def send(self, **kwargs):
        subject = kwargs.get('subject', None)
        message = kwargs.get('message', None)
        recipient_list = kwargs.get('recipient_list', None)

        send_mail(subject=subject,
                  message=message,
                  from_email=settings.DEFAULT_FROM_EMAIL,
                  recipient_list=recipient_list,
                  fail_silently=True)


class EmailMultipleMessageBackend(SyncMessageBackend):
    """
    Send messages through emails on the main thread
    """

    def send(self, datatuples, **kwargs):
        send_mass_mail(
            tuple(tuple(
                d.get('subject', None),
                d.get('message', None),
                settings.DEFAULT_FROM_EMAIL,
                d.get('recipient_list', None))
                for d in datatuples),
            fail_silently=True)
